/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package com.glassfish.owb.dao;

import java.io.Serializable;
import java.util.List;
import javax.ejb.LocalBean;
import javax.ejb.Stateless;
import javax.ejb.TransactionAttribute;
import javax.inject.Named;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.TypedQuery;
import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Root;

/**
 *
 * @author rmpestano Dec 11, 2011 11:14:29 AM
 */
@Stateless
@LocalBean
@Named
public class JPADao<T,Id extends Serializable>{

    @PersistenceContext(unitName="testPU")
    private EntityManager entityManaer;
    
    private Class<T> persistentClass;
    
     /**
     * set by the serviceLayer
     * @param persistentClass 
     */
    public void setPersistentClass(Class<T> persistentClass) {
        this.persistentClass = persistentClass;
    }

    public T load(Id id) {
        return (T) this.entityManaer.find(this.persistentClass, id);
    }

    public T get(Id id) {
        return (T) this.entityManaer.find(this.persistentClass, id);
    }

    @TransactionAttribute
    public void save(T entity) {
        this.entityManaer.persist(entity);
    }

    @TransactionAttribute
    public void update(T entity) {
        this.entityManaer.merge(entity);
    }

    @TransactionAttribute
    public void delete(T entity) {
        this.entityManaer.remove(entity);
    }

    @TransactionAttribute
    public T refresh(T entity) {
        this.entityManaer.refresh(entity);
        return entity;
    }

    @TransactionAttribute
    public void saveOrUpdate(T entity) {
        this.entityManaer.merge(entity);
    }
    
    public List findAll() {
        return getTypedQuery().getResultList();
    }
    
     public CriteriaBuilder getCriteriaBuilder(){
        return  entityManaer.getCriteriaBuilder();
    }
    public CriteriaQuery<T> getCriteriaQuery(){
        return  entityManaer.getCriteriaBuilder().createQuery(persistentClass);
    }
    
    public TypedQuery<T> getTypedQuery(){
        return entityManaer.createQuery(getCriteriaQuery());
    }
    public TypedQuery<T> getTypedQuery(CriteriaQuery<T> criteria){
        return entityManaer.createQuery(criteria);
    }
    public Root<T> getRoot(CriteriaQuery<T> criteria){
        return criteria.from(criteria.getResultType());
    }

    public EntityManager getEntityManaer() {
        return entityManaer;
    }

     
}
